/*
********************************************************************************
     Copyright (c) 2005 Iowa State University, Glenn Luecke, James Coyle, James Hoekstra, Marina Kraeva, Olga Taborskaia, Andre Wehe, Ying Xu, and Ziyu Zhang, All rights reserved.  Licensed under the Educational Community License version 1.0.  See the full agreement at http://rted.public.iastate.edu.
  
     Name of the test:  C_G_1_4_a_d.cpp

     Summary:           use delete operator with an in a union declared pointer that has been modified by changing another variable of the union

     Test description:  the union with the pointer is a default member variable; the allocation is in the constructor; a pointer copy to a global pointer and the indirect changing of the pointer address are in a member function; the deallocation is in the destructor; "double" is used for the base datatype of the pointer

     Error line:        65

     Support files:     Not needed

     Env. requirements: Not needed

     Keywords:          constructor, member function, changed pointer, deallocate, default member variable, destructor, memory address

     Last modified:     07/08/2005 15:27

     Programmer:        Andre Wehe, Iowa State University

********************************************************************************
*/

#include <iostream.h>
#include <stdlib.h>

// this is the declaration of a pointer to prevent a memory leak
double *ptrBackup;

#ifndef EXITCODE_OK
#define EXITCODE_OK 1
#endif

class TestClass {
        // this is the declaration of the pointer in a union
        union UnionA {
            double *ptrA;
            int valueA;
        } unionA;

    public:
        TestClass();
        ~TestClass();
        void runA();
};

TestClass :: TestClass() {
    // allocate memory
    cout << "allocate memory" << endl;
    unionA.ptrA=new double;

    // initialize ptrA
    *unionA.ptrA=rand();
}

TestClass :: ~TestClass() {
    // outsmart dead code elimination
    cout << "ptrA " << &unionA.ptrA << ", " << unionA.ptrA << endl;
    cout << "ptrBackup " << &ptrBackup << ", " << ptrBackup << ", " << *ptrBackup << endl;

    // deallocate memory
    cout << "deallocate memory" << endl;
    delete unionA.ptrA;
}

void TestClass :: runA() {
    // store the value of ptrA to prevent a memory leak
    ptrBackup=unionA.ptrA;

    // outsmart dead code elimination
    cout << "ptrA " << &unionA.ptrA << ", " << unionA.ptrA << ", " << *unionA.ptrA << endl;
    cout << "ptrBackup " << &ptrBackup << ", " << ptrBackup << ", " << *ptrBackup << endl;

    // modify the pointer by incrementing its value
    cout << "modify pointer" << endl;
    unionA.valueA=23;
}

int main() {
    TestClass testclassA;
    testclassA.runA();
    return EXITCODE_OK;
}
