/*
********************************************************************************
     Copyright (c) 2005 Iowa State University, Glenn Luecke, James Coyle, James Hoekstra, Marina Kraeva, Olga Taborskaia, Andre Wehe, Ying Xu, and Ziyu Zhang, All rights reserved.  Licensed under the Educational Community License version 1.0.  See the full agreement at http://rted.public.iastate.edu.
  
     Name of the test:  C_G_2_3_a_d.cpp

     Summary:           don't deallocate the 1st dimension of a pointer array

     Test description:  the pointer to the array of pointers is a default member variable; the allocations for both dimensions are in the constructor; a pointer copy to a global pointer is in a member function; the deallocation for the 2nd dimension is in the destructor; "double" is used for the datatype of the array

     Error line:        

     Support files:     Not needed

     Env. requirements: Not needed

     Keywords:          constructor, member function, 1st dimension, deallocate, default member variable, destructor, memory consumption, missing delete, multidimensional array

     Last modified:     07/08/2005 15:27

     Programmer:        Andre Wehe, Iowa State University

********************************************************************************
*/

#include <iostream.h>
#include <stdlib.h>

#ifndef NCOLUMNS
#define NCOLUMNS 100
#endif

#ifndef NROWS
#define NROWS 123
#endif

// this is the declaration of a pointer to prevent a memory leak
double **ptrBackup;

#ifndef EXITCODE_OK
#define EXITCODE_OK 1
#endif

class TestClass {
        // this is the declaration of the multi dimensional array
        double **arrayA;

    public:
        TestClass();
        ~TestClass();
        void runA();
};

TestClass :: TestClass() {
    // Allocate "main" array
    cout << "allocate memory for the 1st dimension of arrayA" << endl;
    arrayA=new double*[NROWS];

    // Allocate each member of the "main" array
    cout << "allocate memory for the 2nd dimension of arrayA" << endl;
    for (int i=0; i < NROWS; ++i) {
        arrayA[i]=new double[NCOLUMNS];
    }

    // initialize the array
    cout << "fill array with random values" << endl;
    for ( int i=0; i < NROWS; i++) {
        for ( int j=0; j < NCOLUMNS; j++) {
            arrayA[i][j]=rand();
        }
    }
}

TestClass :: ~TestClass() {
    // Delete each member of the "main" array
    cout << "deallocate the memory of the 2nd dimension of arrayA" << endl;
    for (int i=0; i < NROWS; ++i) {
        delete[] arrayA[i];
    }
}

void TestClass :: runA() {
    // store the value of ptrA to prevent a memory leak
    ptrBackup=arrayA;

    // print out ptrA to prevent dead code elimination
    bool value=false;
    for ( int i=0; i < NROWS; i++) {
        for ( int j=0; j < NCOLUMNS; j++) {
            value=value && arrayA[i][j];
        }
    }
    cout << value << endl;
}

int main() {
    TestClass testclassA;
    testclassA.runA();
    return EXITCODE_OK;
}
