/*
********************************************************************************
     Copyright (c) 2005 Iowa State University, Glenn Luecke, James Coyle, James Hoekstra, Marina Kraeva, Olga Taborskaia, Andre Wehe, Ying Xu, and Ziyu Zhang, All rights reserved.  Licensed under the Educational Community License version 1.0.  See the full agreement at http://rted.public.iastate.edu.
  
     Name of the test:  C_G_2_4_a_d.cpp

     Summary:           interrupt the program by calling "exit" without deallocating previously allocated memory

     Test description:  the pointer is a default member variable; the allocation is in the constructor; the "exit" command is in a member function; the never executed deallocation is in the destructor; "double" is used for the base datatype of the pointer

     Error line:        67

     Support files:     Not needed

     Env. requirements: Not needed

     Keywords:          constructor, member function, deallocate, default member variable, destructor, exit, interrupt, memory consumption, missing delete, terminate

     Last modified:     07/08/2005 15:27

     Programmer:        Andre Wehe, Iowa State University

********************************************************************************
*/

#include <iostream.h>
#include <stdlib.h>

// this is the declaration of a pointer to prevent a memory leak
double *ptrBackup;

#ifndef EXITCODE_OK
#define EXITCODE_OK 1
#endif

class TestClass {
        // this is the declaration of the pointer
        double *ptrA;

    public:
        TestClass();
        ~TestClass();
        void runA();
};

TestClass :: TestClass() {
    // allocate memory
    cout << "allocate memory" << endl;
    ptrA=new double;

    // initialize ptrA
    *ptrA=rand();
}

TestClass :: ~TestClass() {
    // deallocate memory
    cout << "deallocate memory" << endl;
    delete ptrA;
}

void TestClass :: runA() {
    // outsmart dead code elimination
    cout << "ptrA " << &ptrA << ", " << ptrA << ", " << *ptrA << endl;

    // quit program without deallocating ptrA
    cout << "exit without deallocating ptrA" << endl;
    exit(EXITCODE_OK);
}

int main() {
    TestClass testclassA;
    testclassA.runA();
    return EXITCODE_OK;
}
