/*
********************************************************************************
     Copyright (c) 2005 Iowa State University, Glenn Luecke, James Coyle, James Hoekstra, Marina Kraeva, Olga Taborskaia, Andre Wehe, Ying Xu, and Ziyu Zhang, All rights reserved.  Licensed under the Educational Community License version 1.0.  See the full agreement at http://rted.public.iastate.edu.
  
     Name of the test:  C_E_1_7_k_ld.cpp

     Summary:           read several elements past the right bound

     Test description:  the multidimensional array is a public member variable of another class; the read is in a member function; "long double" is used for the datatype of the array

     Error line:        78

     Support files:     Not needed

     Env. requirements: Not needed

     Keywords:          member function, public member variable, class, multidimensional array, out-of-bounds, read, right bound

     Last modified:     07/08/2005 15:27

     Programmer:        Andre Wehe, Iowa State University

********************************************************************************
*/

#include <iostream.h>
#include <stdlib.h>

#ifndef NCOLUMNS
#define NCOLUMNS 100
#endif

#ifndef NROWS
#define NROWS 123
#endif

#ifndef GAP
#define GAP 50
#endif

#ifndef EXITCODE_OK
#define EXITCODE_OK 1
#endif

class TestClassData {
    public:
        // this is the declaration of the multi dimensional array
        long double arrayA[NROWS][NCOLUMNS];
};

TestClassData classdataA;

class TestClass {
    public:
        void runA();
};

void TestClass :: runA() {
    // initialize the array
    cout << "fill array with random values" << endl;
    for ( int i=0; i < NROWS; i++) {
        for ( int j=0; j < NCOLUMNS; j++) {
            classdataA.arrayA[i][j]=rand();
        }
    }

    // outsmart dead code elimination
    int i=NROWS - 1;
    int j=NCOLUMNS - 1 + GAP;
    if ( 10.0 * rand() / (RAND_MAX + 1.0) > 10) {
        cout << "this should never be called" << endl;
        i=NROWS * rand();
        j=NCOLUMNS * rand();
    }

    // read several elements past the right bound
    cout << "read array[" << i << "][" << j << "]" << endl;
    long double value=classdataA.arrayA[i][j];

    // print out ptrA to prevent dead code elimination
    for ( int i=0; i < NROWS; i++) {
        for ( int j=0; j < NCOLUMNS; j++) {
            value=value && classdataA.arrayA[i][j];
        }
    }
    cout << value << endl;
}

int main() {
    TestClass testclassA;
    testclassA.runA();
    return EXITCODE_OK;
}
