/*
********************************************************************************
     Copyright (c) 2005 Iowa State University, Glenn Luecke, James Coyle, James Hoekstra, Marina Kraeva, Olga Taborskaia, Andre Wehe, Ying Xu, and Ziyu Zhang, All rights reserved.  Licensed under the Educational Community License version 1.0.  See the full agreement at http://rted.public.iastate.edu.
  
     Name of the test:  C_E_2_7_h_ld.cpp

     Summary:           read several elements past the right bound of the first dimension

     Test description:  the pointer to the multidimensional array is a "static" member variable; the allocation is in the constructor; the read is in a member function; "long double" is used for the datatype of the array

     Error line:        99

     Support files:     Not needed

     Env. requirements: Not needed

     Keywords:          constructor, member function, static member variable, allocated memory, destructor, multidimensional array, out-of-bounds, read, right bound

     Last modified:     07/08/2005 15:27

     Programmer:        Andre Wehe, Iowa State University

********************************************************************************
*/

#include <iostream.h>
#include <stdlib.h>

#ifndef NCOLUMNS
#define NCOLUMNS 100
#endif

#ifndef NROWS
#define NROWS 123
#endif

#ifndef GAP
#define GAP 50
#endif

#ifndef EXITCODE_OK
#define EXITCODE_OK 1
#endif

class TestClass {
        // this is the declaration of the multi dimensional array
        static long double**arrayA;

    public:
        TestClass();
        ~TestClass();
        void runA();
};

long double**TestClass::arrayA;

TestClass :: TestClass() {
    // Allocate "main" array
    cout << "allocate memory for arrayA" << endl;
    arrayA=new long double*[NROWS];

    // Allocate each member of the "main" array
    for (int i=0; i < NROWS; ++i) {
        arrayA[i]=new long double[NCOLUMNS];
    }
}

TestClass :: ~TestClass() {
    // Delete each member of the "main" array
    cout << "deallocate the memory of arrayA" << endl;
    for (int i=0; i < NROWS; ++i) {
        delete[] arrayA[i];
    }

    // Delete "main" array
    delete[] arrayA;
}

void TestClass :: runA() {
    // initialize the array
    cout << "fill array with random values" << endl;
    for ( int i=0; i < NROWS; i++) {
        for ( int j=0; j < NCOLUMNS; j++) {
            arrayA[i][j]=rand();
        }
    }

    // outsmart dead code elimination
    int i=NROWS - 1 + GAP;
    int j=NCOLUMNS - 1;
    if ( 10.0 * rand() / (RAND_MAX + 1.0) > 10) {
        cout << "this should never be called" << endl;
        i=NROWS * rand();
        j=NCOLUMNS * rand();
    }

    // read several elements past the right bound
    cout << "read array[" << i << "][" << j << "]" << endl;
    long double value=arrayA[i][j];

    // print out ptrA to prevent dead code elimination
    for ( int i=0; i < NROWS; i++) {
        for ( int j=0; j < NCOLUMNS; j++) {
            value=value && arrayA[i][j];
        }
    }
    cout << value << endl;
}

int main() {
    TestClass testclassA;
    testclassA.runA();
    return EXITCODE_OK;
}
