/*
********************************************************************************
     Copyright (c) 2005 Iowa State University, Glenn Luecke, James Coyle, James Hoekstra, Marina Kraeva, Olga Taborskaia, Andre Wehe, Ying Xu, and Ziyu Zhang, All rights reserved.  Licensed under the Educational Community License version 1.0.  See the full agreement at http://rted.public.iastate.edu.
  
     Name of the test:  C_E_2_7_i_f.cpp

     Summary:           read several elements past the right bound of the first dimension

     Test description:  the pointer to the multidimensional array is a public member variable of the class "TestClassData"; the allocation is in the constructor of the class "TestClassData"; the read is in a member function of the class "TestClass"; "float" is used for the datatype of the array

     Error line:        102

     Support files:     Not needed

     Env. requirements: Not needed

     Keywords:          class, constructor, member function, public member variable, allocated memory, destructor, multidimensional array, out-of-bounds, read, right bound

     Last modified:     07/08/2005 15:27

     Programmer:        Andre Wehe, Iowa State University

********************************************************************************
*/

#include <iostream.h>
#include <stdlib.h>

#ifndef NCOLUMNS
#define NCOLUMNS 100
#endif

#ifndef NROWS
#define NROWS 123
#endif

#ifndef GAP
#define GAP 50
#endif

#ifndef EXITCODE_OK
#define EXITCODE_OK 1
#endif

class TestClassData {
    public:
        TestClassData();
        ~TestClassData();
        // this is the declaration of the multi dimensional array
        float **arrayA;
};

TestClassData :: TestClassData() {
    // Allocate "main" array
    cout << "allocate memory for arrayA" << endl;
    arrayA=new float*[NROWS];

    // Allocate each member of the "main" array
    for (int i=0; i < NROWS; ++i) {
        arrayA[i]=new float[NCOLUMNS];
    }
}

TestClassData :: ~TestClassData() {
    // Delete each member of the "main" array
    cout << "deallocate the memory of arrayA" << endl;
    for (int i=0; i < NROWS; ++i) {
        delete[] arrayA[i];
    }

    // Delete "main" array
    delete[] arrayA;
}

TestClassData classdataA;

class TestClass {
    public:
        void runA();
};

void TestClass :: runA() {
    // initialize the array
    cout << "fill array with random values" << endl;
    for ( int i=0; i < NROWS; i++) {
        for ( int j=0; j < NCOLUMNS; j++) {
            classdataA.arrayA[i][j]=rand();
        }
    }

    // outsmart dead code elimination
    int i=NROWS - 1 + GAP;
    int j=NCOLUMNS - 1;
    if ( 10.0 * rand() / (RAND_MAX + 1.0) > 10) {
        cout << "this should never be called" << endl;
        i=NROWS * rand();
        j=NCOLUMNS * rand();
    }

    // read several elements past the right bound
    cout << "read array[" << i << "][" << j << "]" << endl;
    float value=classdataA.arrayA[i][j];

    // print out ptrA to prevent dead code elimination
    for ( int i=0; i < NROWS; i++) {
        for ( int j=0; j < NCOLUMNS; j++) {
            value=value && classdataA.arrayA[i][j];
        }
    }
    cout << value << endl;
}

int main() {
    TestClass testclassA;
    testclassA.runA();
    return EXITCODE_OK;
}
