/*
********************************************************************************
     Copyright (c) 2005 Iowa State University, Glenn Luecke, James Coyle, James Hoekstra, Marina Kraeva, Olga Taborskaia, Andre Wehe, Ying Xu, and Ziyu Zhang, All rights reserved.  Licensed under the Educational Community License version 1.0.  See the full agreement at http://rted.public.iastate.edu.
  
     Name of the test:  C_H_2_3_a_fs.cpp

     Summary:           write to a file that is opened for input

     Test description:  the file stream is a default member variable; the open is in the constructor; the write is in a member function; the close is in the destructor; "fstream" is used for the file stream

     Error line:        77

     Support files:     Not needed

     Env. requirements: Write and read access to create a file for testing

     Keywords:          constructor, input, member function, write, default member variable, destructor, file stream

     Last modified:     07/08/2005 15:27

     Programmer:        Andre Wehe, Iowa State University

********************************************************************************
*/

#include <iostream.h>
#include <stdlib.h>
#include <fstream.h>

#ifndef FILENAME
#define FILENAME "fileA.txt"
#endif

#ifndef TESTSTRING
#define TESTSTRING "hello world"
#endif

#ifndef EXITCODE_ERROR
#define EXITCODE_ERROR 2
#endif

#ifndef EXITCODE_OK
#define EXITCODE_OK 1
#endif

class TestClass {
        // declaration of the file stream
        fstream streamA;

    public:
        TestClass();
        ~TestClass();
        void runA();
};

TestClass :: TestClass() {
    // create a file labeled as FILENAME for later use
    ofstream streamB(FILENAME, ios::out);
    if (!streamB) {
        cerr << "Cannot open file " << FILENAME << "." << endl;
        exit(EXITCODE_ERROR);
    }
    streamB << TESTSTRING << endl;
    streamB.close();

    // open the file stream
    streamA.open(FILENAME, ios::in);
    if (!streamA) {
        cerr << "Cannot open file " << FILENAME << "." << endl;
        exit(EXITCODE_ERROR);
    }
}

TestClass :: ~TestClass() {
    // try to write to this file stream
    cout << "write " << TESTSTRING << " to the file " << FILENAME << endl;
    streamA << TESTSTRING;
}

void TestClass :: runA() {
    // close the file stream
    streamA.close();
}

int main() {
    TestClass testclassA;
    testclassA.runA();
    return EXITCODE_OK;
}
