/*
********************************************************************************
     Copyright (c) 2005 Iowa State University, Glenn Luecke, James Coyle, James Hoekstra, Marina Kraeva, Olga Taborskaia, Andre Wehe, Ying Xu, and Ziyu Zhang, All rights reserved.  Licensed under the Educational Community License version 1.0.  See the full agreement at http://rted.public.iastate.edu.
  
     Name of the test:  C_H_3_4_a_fs.cpp

     Summary:           move the position in an already closed file using seekp

     Test description:  the file stream is a default member variable; the open and close are in the constructor; the move is in a member function; "fstream" is used for the file stream

     Error line:        93

     Support files:     Not needed

     Env. requirements: Write and read access to create a file for testing

     Keywords:          close, member function, move the position, constructor, default member variable, unopened file stream

     Last modified:     07/08/2005 15:27

     Programmer:        Andre Wehe, Iowa State University

********************************************************************************
*/

#include <iostream.h>
#include <stdlib.h>
#include <fstream.h>

#ifndef FILENAME
#define FILENAME "fileA.txt"
#endif

#ifndef TESTSTRING
#define TESTSTRING "hello world"
#endif

#ifndef FILEPOSISTION
#define FILEPOSISTION 5
#endif

#ifndef EXITCODE_ERROR
#define EXITCODE_ERROR 2
#endif

#ifndef EXITCODE_OK
#define EXITCODE_OK 1
#endif

class TestClass {
        // declaration of the file stream
        fstream streamA;

    public:
        TestClass();
        void runA();
};

TestClass :: TestClass() {
    // create a file for later use
    ofstream streamB(FILENAME, ios::out);
    if (!streamB) {
        cerr << "Cannot open file " << FILENAME << "." << endl;
        exit(EXITCODE_ERROR);
    }
    streamB << TESTSTRING << endl;
    streamB.close();

    // open the file stream
    streamA.open(FILENAME, ios::out);
    if (!streamA) {
        cerr << "Cannot open file " << FILENAME << "." << endl;
        exit(EXITCODE_ERROR);
    }

    // close the file stream
    streamA.close();
}

void TestClass :: runA() {
    // outsmart dead code elimination
    if ( 10.0 * rand() / (RAND_MAX + 1.0) > 10) {
        cout << "this should never be called" << endl;
        streamA.open(FILENAME, ios::in);
        if (!streamA) {
            cerr << "Cannot open file " << FILENAME << "." << endl;
            exit(EXITCODE_ERROR);
        }
    }

    // try to move the reading/writing position
    cout << "move position of the file " << FILENAME << endl;
    streamA.seekp(FILEPOSISTION, ios::beg);

    // outsmart dead code elimination
    if ( 10.0 * rand() / (RAND_MAX + 1.0) > 10) {
        cout << "this should never be called" << endl;
        streamA.close();
    }
}

int main() {
    TestClass testclassA;
    testclassA.runA();
    return EXITCODE_OK;
}
