/*
********************************************************************************
     Copyright (c) 2005 Iowa State University, Glenn Luecke, James Coyle, James Hoekstra, Marina Kraeva, Olga Taborskaia, Andre Wehe, Ying Xu, and Ziyu Zhang, All rights reserved.  Licensed under the Educational Community License version 1.0.  See the full agreement at http://rted.public.iastate.edu.
  
     Name of the test:  C_I_2_5_a_d.cpp

     Summary:           non-execution of a destructor by using a base class pointer

     Test description:  the pointer is a default member variable in the derived class; the memory allocation is in the constructor of the derived class; the memory deallocation is in the destructor of the derived class; the object of the derived class type is allocated in main function by using a pointer to the base class; "double" is used for the base datatype of the pointer in the derived class.

     Error line:        79

     Support files:     Not needed

     Env. requirements: Not needed

     Keywords:          allocate, base class, constructor, derived, destructor, function, protected member variable, losing pointer

     Last modified:     07/08/2005 15:27

     Programmer:        Andre Wehe, Iowa State University

********************************************************************************
*/

#include <iostream.h>
#include <stdlib.h>

#ifndef EXITCODE_OK
#define EXITCODE_OK 1
#endif

class TestClassBase {
    public:
        TestClassBase() {
            cout<<"TestClassBase Constructor"<<endl;
        };

        ~TestClassBase() {
            cout<<"TestClassBase Destructor"<<endl;
        };

        virtual void runA()=0;
};

class TestClass : public TestClassBase
{
        // this is the declaration of the pointer
        double *ptrA;

    public:
        TestClass() {
            // memory allocation
            cout << "TestClass Constructor" << endl;
            ptrA=new double;

            // initialize
            *ptrA=rand();
        }

        ~TestClass() {
            // memory deallocation
            cout << "TestClass Destructor" << endl;
            delete ptrA;
        }

        void runA();
};

void TestClass :: runA() {
    // outsmart dead code elimination
    cout << "ptrA " << &ptrA << ", " << ptrA << ", " << *ptrA << endl;
}

int main()
{
    TestClassBase *testclassA=new TestClass;
    testclassA->runA();
    delete testclassA;

    return EXITCODE_OK;
}
