/*
********************************************************************************
     Copyright (c) 2005 Iowa State University, Glenn Luecke, James Coyle, James Hoekstra, Marina Kraeva, Olga Taborskaia, Andre Wehe, Ying Xu, and Ziyu Zhang, All rights reserved.  Licensed under the Educational Community License version 1.0.  See the full agreement at http://rted.public.iastate.edu.
  
     Name of the test:  C_I_2_6_b_o.cpp

     Summary:           losing a pointer by catching an error

     Test description:  the pointer declaration, memory allocation, and memory deallocation are in a member function that throws an error; an exception is thrown before the memory deallocation; "TestObject" is used for the base datatype of the pointer.

     Error line:        77

     Support files:     TestObject.h

     Env. requirements: Not needed

     Keywords:          allocate, catch, execption, try, losing pointer

     Last modified:     10/12/2005 15:23

     Programmer:        Andre Wehe, Iowa State University

********************************************************************************
*/

#include "TestObject.h"
#include <iostream.h>
#include <stdlib.h>
#include <vector>
#include <stdexcept>

#ifndef EXITCODE_OK
#define EXITCODE_OK 1
#endif

using namespace std;

template<class T> class VecA : public std::vector<T> {
    public:
        VecA() : vector<T>() {}
        VecA(int value) : vector<T>(value) {}
        T& operator [] (int i) {
            return this->at(i);
        }
        const T& operator [] (int i) const {
            return this->at(i);
        }
};

class TestclassA {
    public:
        void runA() throw (out_of_range) {
            // create a vector with 100 entries
            VecA<TestObject> vecA(100);

            // allocate memory
            TestObject *ptrA;
            ptrA = new TestObject;

            // outsmart dead code elimination
            int indexA=100;
            if ( 10.0 * rand() / (RAND_MAX + 1.0) > 10) {
                cout << "this should never be called" << endl;
                indexA=0;
            }

            // case an out of range exception
            vecA[indexA]=*ptrA;

            // outsmart dead code elimination
            if ( 10.0 * rand() / (RAND_MAX + 1.0) > 10) {
                cout << vecA[indexA] << endl;
            }

            // unfortunately this is never called
            delete ptrA;
        }
};

int main() {
    try {
        TestclassA testclassA;
        testclassA.runA();
    }
    catch (out_of_range) {
        cout << "error caught" << endl;
    }
    return EXITCODE_OK;
}
