/*
********************************************************************************
     Copyright (c) 2005 Iowa State University, Glenn Luecke, James Coyle, James Hoekstra, Marina Kraeva, Olga Taborskaia, Andre Wehe, Ying Xu, and Ziyu Zhang, All rights reserved.  Licensed under the Educational Community License version 1.0.  See the full agreement at http://rted.public.iastate.edu.
  
     Name of the test:  C_F_1_4_a_d.cpp

     Summary:           refer to several elements past the right bound of a statically allocated array

     Test description:  the array is a default member variable; the declaration of the reference is in a member function; "double" is used for the datatype of the array

     Error line:        66

     Support files:     Not needed

     Env. requirements: Not needed

     Keywords:          array, default member variable, member function, out-of-bounds, read, reference, right bound

     Last modified:     07/08/2005 15:27

     Programmer:        Andre Wehe, Iowa State University

********************************************************************************
*/

#include <iostream.h>
#include <stdlib.h>

#ifndef SIZE
#define SIZE 100
#endif

#ifndef GAP
#define GAP 50
#endif

#ifndef EXITCODE_OK
#define EXITCODE_OK 1
#endif

class TestClass {
        // this is the declaration of the array
        double arrayA[SIZE];

    public:
        void runA();
};

void TestClass :: runA() {
    // fill the array with random values
    for ( int i=0; i < SIZE; i++) {
        arrayA[i]=rand();
    }

    // outsmart dead code elimination
    int i=SIZE - 1+ GAP;
    if ( 10.0 * rand() / (RAND_MAX + 1.0) > 10) {
        cout << "this should never be called" << endl;
        i=SIZE * rand();
    }

    // several elements past the right bound
    cout << "array[" << i << "] with bounds 0:" << SIZE-1 << endl;

    // declare a reference to an element of the array
    double & refA=arrayA[i];

    // outsmart dead code elimination
    if ( 10.0 * rand() / (RAND_MAX + 1.0) > 10) {
        double value=refA;
        for ( int i=0; i < SIZE; i++) {
            value=value && arrayA[i];
        }
        cout << value << endl;
    }
}

int main() {
    TestClass testclassA;
    testclassA.runA();
    return EXITCODE_OK;
}
