/*
********************************************************************************
     Copyright (c) 2005 Iowa State University, Glenn Luecke, James Coyle, James Hoekstra, Marina Kraeva, Olga Taborskaia, Andre Wehe, Ying Xu, and Ziyu Zhang, All rights reserved.  Licensed under the Educational Community License version 1.0.  See the full agreement at http://rted.public.iastate.edu.
  
     Name of the test:  C_J_2_1_a_d.cpp

     Summary:           create an object in the stack memory that is bigger than the available stack memory size

     Test description:  the array is a default member variable; the declaration of the object is in a function; the function is called in the main function; "double" is used for the datatype of the array.

     Error line:        84

     Support files:     Not needed

     Env. requirements: The stacksize is set in the program to be smaller than STACKARRAYSIZE * sizeof(double)

     Keywords:          allocate, memory, object, stack

     Last modified:     07/08/2005 15:27

     Programmer:        Andre Wehe, Iowa State University

********************************************************************************
*/

#include <iostream.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <unistd.h>

#ifndef EXITCODE_ERROR
#define EXITCODE_ERROR 2
#endif

#ifndef EXITCODE_OK
#define EXITCODE_OK 1
#endif

#ifndef STACKARRAYSIZE
#define STACKARRAYSIZE 100000
#endif

class TestClass {
        // declaration of the array
        double arrayA[STACKARRAYSIZE];

    public:
        TestClass();
        void runA();
};

TestClass :: TestClass() {
    for (int i=0; i<STACKARRAYSIZE; i++)
        arrayA[i] = rand();
}

void TestClass :: runA() {
    // dead-code elimination
    int indexA=(int) (STACKARRAYSIZE * (rand() / (RAND_MAX + 1.0)));
    cout << "array[" << indexA << "] = " << arrayA[indexA] << endl;
}

int main() {
    // set the stack size resource limit.
    struct rlimit limit;
    limit.rlim_cur = STACKARRAYSIZE * sizeof(double) / 2;
    limit.rlim_max = STACKARRAYSIZE * sizeof(double) / 2;
    if (int err=setrlimit(RLIMIT_STACK, &limit) != 0) {
        cout << "setrlimit() failed with error " << err << endl;
        exit(EXITCODE_ERROR);
    }

    // get the stack size resource limit.
    if (int err=getrlimit(RLIMIT_STACK, &limit) != 0) {
        cout << "getrlimit() failed with error " << err << endl;
        exit(EXITCODE_ERROR);
    }
    cout << "stack soft limit: " << limit.rlim_cur << endl;
    cout << "stack hard limit: " << limit.rlim_max << endl;

    // create an object of TestClass
    cout << "TestClass size: " << sizeof(TestClass) << endl;
    TestClass testclassA;
    testclassA.runA();

    return EXITCODE_OK;
}
